/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-16 15:00:21.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.model;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import java.io.Serializable;

public class Result<T> implements Serializable {
    private static final long serialVersionUID = 1L;
    private String code;//状态码
    private String desc;//描述信息
    private T data;//数据

    /* 常用状态码 */
    public static String CODE_200 = "200";//成功
    public static String CODE_500 = "500";//失败
    public static String CODE_403 = "403";//认证失败
    public static String DESC_SUCCESS = "操作成功";
    public static String DESC_FAIL = "操作失败";

    public Result() {

    }

    public Result(String code, String desc, T data) {
        this.code = code;
        this.desc = desc;
        this.data = data;
    }

    /**
     * 成功结果
     */
    public Result success(T data) {
        this.code = CODE_200;
        this.desc = DESC_SUCCESS;
        this.data = data;
        return this;
    }

    public Result success() {
        this.code = CODE_200;
        this.desc = DESC_SUCCESS;
        return this;
    }

    /**
     * 错误结果
     */
    public Result error(String desc) {
        this.code = CODE_500;
        this.desc = desc;
        return this;
    }

    public Result error() {
        this.code = CODE_500;
        this.desc = DESC_FAIL;
        return this;
    }

    public Result tokenError() {
        this.code = CODE_403;
        this.desc = "No login,Token is invalid.";
        return this;
    }

    public String getCode() {
        return code;
    }

    public String getDesc() {
        return desc;
    }

    public T getData() {
        return data;
    }

    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this, ToStringStyle.JSON_STYLE);
    }

}